@@ -1,4 +1,4 @@
-# Copyright 1998-1999, Paul Johnson (pjcj@transeda.com)
+# Copyright 1998-2000, Paul Johnson (pjcj@cpan.org)
 
 # This software is free.  It is licensed under the same terms as Perl itself.
 
@@ -13,13 +13,13 @@ require 5.005;
 
 package Gedcom::Record;
 
-use Carp;
+use Carp ();
 BEGIN { eval "use Date::Manip" }             # We'll use this if it is available
 
-use Gedcom::Item 1.05;
+use Gedcom::Item 1.06;
 
 use vars qw($VERSION @ISA $AUTOLOAD);
-$VERSION = "1.05";
+$VERSION = "1.06";
 @ISA     = qw( Gedcom::Item );
 
 my %Funcs;
@@ -32,64 +32,140 @@ BEGIN
     if ($name)
     {
       $name =~ s/ /_/g;
-      $name = lc $name;
       $Funcs{lc $name} = $tag;
     }
   }
   # use Data::Dumper;
   # print "Funcs are ", Dumper(\%Funcs);
   use subs keys %Funcs;
+  *tag_record    = \&Gedcom::Item::get_item;
+  *delete_record = \&Gedcom::Item::delete_item;
+  *get_record    = \&record;
 }
 
 sub AUTOLOAD
 {
-  my $self = shift;
+  my ($self) = @_;                         # don't change @_ because of the goto
   return if $AUTOLOAD =~ /::DESTROY$/;
   my $func = $AUTOLOAD;
   # print "autoloading $func\n";
   $func =~ s/^.*:://;
-  my $child = $Funcs{lc $func};
-  croak "Undefined subroutine $func called" unless $child;
+  warn "Undefined subroutine $func called" unless $Funcs{lc $func};
+  no strict "refs";
+  *$func = sub
+  {
+    my $self = shift;
+    my ($count) = @_;
+    if (wantarray)
+    {
+      return map { $_ && $_->{value} || $_ } $self->record([$func, $count]);
+    }
+    else
+    {
+      my $record = $self->record([$func, $count]);
+      return $record && $record->{value} || $record;
+    }
+  };
+  goto &$func
+}
+
+sub record
+{
+  my $self = shift;
+  my @records = ($self);
+  for my $func (map { ref() ? $_ : split } @_)
+  {
+    my $count = 0;
+    ($func, $count) = @$func if ref $func eq "ARRAY";
+    if (ref $func)
+    {
+      warn "Invalid record of type ", ref $func, " requested";
+      return undef;
+    }
+    my $record = $Funcs{lc $func};
+    unless ($record)
+    {
+      warn $func
+      ? "Non standard record of type $func requested"
+      : "Record type not specified";
+      $record = $func;
+    }
+    @records = map { $_->tag_record($record, $count) } @records;
+  }
+  wantarray ? @records : $records[0]
+}
+
+sub set_record
+{
+  my $self = shift;
+  my $new_record = pop;
+  my $last_record = pop;
+  my $r = $self->record(@_);
+  unless ($r)
+  {
+    warn "no record found";
+    return;
+  }
+  my ($record, $count) = parse_func($last_record);
+}
+
+sub get_value
+{
+  my $self = shift;
   if (wantarray)
   {
-    my @c = $self->get_children($child);
-    return map { $_ && $_->{value} ? $_->{value} : $_ } @c;
+    return map { $_->{value} || () } $self->record(@_);
   }
   else
   {
-    my $c = $self->get_child($child);
-    return $c && $c->{value} ? $c->{value} : $c;
+    my $record = $self->record(@_);
+    return $record && $record->{value};
+  }
+}
+
+sub tag_value
+{
+  my $self = shift;
+  if (wantarray)
+  {
+    return map { $_->{value} || () } $self->tag_record(@_);
+  }
+  else
+  {
+    my $record = $self->tag_record(@_);
+    return $record && $record->{value};
   }
 }
 
 sub parse
 {
+# print "parsing\n";
   my $self = shift;
   my ($record, $grammar) = @_;
-  # print "checking "; $self->print();
-  # print "against ";  $grammar->print();
+# print "checking "; $self->print();
+# print "against ";  $grammar->print();
   my $t = $record->{tag};
   my $g = $grammar->{tag};
   die "Can't match $t with $g" if $t && $t ne $g;               # internal error
   $record->{grammar} = $grammar;
   my $class = $record->{gedcom}{types}{$t};
-# print "$t is a $class\n" if $class;
   bless $record, "Gedcom::$class" if $class;
-  for my $child (@{$record->{children}})
+  for my $r (@{$record->{items}})
   {
-    my $tag = $child->{tag};
-    if (my $gc = $grammar->child($tag))
+    my $tag = $r->{tag};
+    if (my $i = $grammar->item($tag))
     {
-      $self->parse($child, $gc);
+      $self->parse($r, $i);
     }
     else
     {
-      warn "$self->{file}:$child->{line}: $tag is not a child of $t\n"
+      warn "$self->{file}:$r->{line}: $tag is not a sub-item of $t\n"
         unless substr($tag, 0, 1) eq "_";
         # unless $tag eq "CONT" || $tag eq "CONC" || substr($tag, 0, 1) eq "_";
         # TODO - should CONT and CONC be allowed anywhere?
     }
   }
+# print "parsed\n";
 }
 
 sub collect_xrefs
@@ -97,7 +173,7 @@ sub collect_xrefs
   my $self = shift;
   my ($callback) = @_;;
   $self->{gedcom}{xrefs}{$self->{xref}} = $self if defined $self->{xref};
-  $_->collect_xrefs($callback) for @{$self->{children}};
+  $_->collect_xrefs($callback) for @{$self->{items}};
   $self
 }
 
@@ -121,7 +197,7 @@ sub resolve_xrefs
   {
     $self->{value} = $xref;
   }
-  $_->resolve_xrefs($callback) for @{$self->{children}};
+  $_->resolve_xrefs($callback) for @{$self->_items};
   $self
 }
 
@@ -133,7 +209,7 @@ sub unresolve_xrefs
     if defined $self->{value}
        and UNIVERSAL::isa $self->{value}, "Gedcom::Record"
        and exists $self->{value}{xref};
-  $_->unresolve_xrefs($callback) for @{$self->{children}};
+  $_->unresolve_xrefs($callback) for @{$self->_items};
   $self
 }
 
@@ -154,18 +230,18 @@ sub validate_syntax
   my $here = "$file:$self->{line}: $self->{tag}" .
              (defined $self->{xref} ? " $self->{xref}" : "");
   my %counts;
-  for my $child (@{$self->{children}})
+  for my $record (@{$self->_items})
   {
-    print "  " x $I . "level $child->{level} on $self->{level}\n" if $D;
-    $ok = 0, warn "$here: Can't add level $child->{level} to $self->{level}\n"
-      if $child->{level} > $self->{level} + 1;
-    $counts{$child->{tag}}++;
-    $ok = 0 unless $child->validate_syntax;
+    print "  " x $I . "level $record->{level} on $self->{level}\n" if $D;
+    $ok = 0, warn "$here: Can't add level $record->{level} to $self->{level}\n"
+      if $record->{level} > $self->{level} + 1;
+    $counts{$record->{tag}}++;
+    $ok = 0 unless $record->validate_syntax;
   }
-  my $valid_children = $grammar->valid_children;
-  for my $tag (sort keys %$valid_children)
+  my $valid_items = $grammar->valid_items;
+  for my $tag (sort keys %$valid_items)
   {
-    my $g = $valid_children->{$tag};
+    my $g = $valid_items->{$tag};
     my $min = $g->{min};
     my $max = $g->{max};
     my $matches = delete $counts{$tag} || 0;
@@ -176,9 +252,9 @@ sub validate_syntax
   }
   for my $tag (keys %counts)
   {
-    for my $c ($self->get_children($tag))
+    for my $c ($self->tag_record($tag))
     {
-      $ok = 0, warn "$file:$c->{line}: $tag is not a child of $self->{tag}\n"
+      $ok = 0, warn "$file:$c->{line}: $tag is not a sub-item of $self->{tag}\n"
         unless substr($tag, 0, 1) eq "_";
         # unless $tag eq "CONT" || $tag eq "CONC" || substr($tag, 0, 1) eq "_";
         # TODO - should CONT and CONC be allowed anywhere?
@@ -188,66 +264,62 @@ sub validate_syntax
   $ok;
 }
 
+my $Check =
+{
+  INDI =>
+  {
+    FAMS => [ "HUSB", "WIFE" ],
+    FAMC => [ "CHIL" ]
+  },
+  FAM =>
+  {
+    HUSB => [ "FAMS" ],
+    WIFE => [ "FAMS" ],
+    CHIL => [ "FAMC" ],
+  },
+};
+
 sub validate_semantics
 {
   my $self = shift;
   return 1 unless $self->{tag} eq "INDI" || $self->{tag} eq "FAM";
   # print "validating: "; $self->print; print $self->summary, "\n";
   my $ok = 1;
-  my $file = $self->{gedcom}{record}{file};
   my $xrefs = $self->{gedcom}{xrefs};
-  my $found;
-  my $child;
-  my $check =
-  {
-    INDI =>
-    {
-      FAMS => [ "HUSB", "WIFE" ],
-      FAMC => [ "CHIL" ]
-    },
-    FAM =>
-    {
-      HUSB => [ "FAMS" ],
-      WIFE => [ "FAMS" ],
-      CHIL => [ "FAMC" ],
-    },
-  };
-  my $chk = $check->{$self->{tag}};
+  my $chk = $Check->{$self->{tag}};
   for my $f (keys %$chk)
   {
-    $found = 1;
-    CHILD:
-    for $child ($self->child_values($f))
+    my $found = 1;
+    RECORD:
+    for my $record ($self->tag_value($f))
     {
       $found = 0;
-      $child = $xrefs->{$child} unless ref $child;
-      if ($child)
+      $record = $xrefs->{$record} unless ref $record;
+      if ($record)
       {
         for my $back (@{$chk->{$f}})
         {
           # print "back $back\n";
-          for my $ch ($child->child_values($back))
+          for my $i ($record->tag_value($back))
           {
-            # print "child is $ch\n";
-            $ch = $xrefs->{$ch} unless ref $ch;
-            if ($ch)
+            # print "record is $i\n";
+            $i = $xrefs->{$i} unless ref $i;
+            if ($i && $i->{xref} eq $self->{xref})
             {
-              if ($ch->{xref} eq $self->{xref})
-              {
-                $found = 1;
-                # print "found...\n";
-                next CHILD;
-              }
+              $found = 1;
+              # print "found...\n";
+              next RECORD;
             }
           }
         }
         unless ($found)
         {
-          # TODO - use the line of the offending child
+          # TODO - use the line of the offending record
           $ok = 0;
-          warn "$file:$self->{line}: $f $child->{xref} " .
+          my $file = $self->{gedcom}{record}{file};
+          warn "$file:$self->{line}: $f $record->{xref} " .
                "does not reference $self->{tag} $self->{xref}. Add the line:\n".
-               "$file:" . ($child->{line} + 1) . ": 1   " .
+               "$file:" . ($record->{line} + 1) . ": 1   " .
                join("or ", @{$chk->{$f}}) .  " $self->{xref}\n";
         }
       }
@@ -261,7 +333,7 @@ sub normalise_dates
   my $self = shift;
   unless ($INC{"Date/Manip.pm"})
   {
-    warn "Date::Manip is required to use normalise_dates()";
+    warn "Date::Manip.pm is required to use normalise_dates()";
     return;
   }
   my $format = shift || "%A, %E %B %Y";
@@ -285,7 +357,8 @@ sub normalise_dates
       # print "date is  $self->{value}\n";
     }
   }
-  $_->normalise_dates($format) for @{$self->{children}};
+  $_->normalise_dates($format) for @{$self->_items};
+  $self->delete_items if $self->level > 1;
 }
 
 sub renumber
@@ -309,17 +382,16 @@ sub renumber
 
 sub child_value
 {
+  # NOTE - This function is deprecated - use tag_value instead
   my $self = shift;;
-  my ($child) = @_;
-  my $c = $self->get_child($child);
-  $c ? $c->{value} : undef;
+  $self->tag_value(@_)
 }
 
 sub child_values
 {
+  # NOTE - This function is deprecated - use tag_value instead
   my $self = shift;;
-  my ($child) = @_;
-  map { $_->{value} } $self->get_children($child);
+  $self->tag_value(@_)
 }
 
 sub summary
@@ -327,13 +399,12 @@ sub summary
   my $self = shift;
   my $s = "";
   $s .= sprintf("%-5s", $self->{xref});
-  my $child = $self->get_child("NAME");
-  $s .= sprintf(" %-40s", $child ? $child->{value} : "");
-  $child = $self->get_child("SEX");
-  $s .= sprintf(" %1s", $child ? $child->{value} : "");
+  my $r = $self->tag_record("NAME");
+  $s .= sprintf(" %-40s", $r ? $r->{value} : "");
+  $r = $self->tag_record("SEX");
+  $s .= sprintf(" %1s", $r ? $r->{value} : "");
   my $d = "";
-  if ($child   = $self->get_child("BIRT") and
-      my $date = $child->get_child("DATE"))
+  if ($r = $self->tag_record("BIRT") and my $date = $r->tag_record("DATE"))
   {
     $d = $date->{value};
   }
@@ -347,26 +418,37 @@ __END__
 
 =head1 NAME
 
-Gedcom::Record - a class to manipulate Gedcom records
+Gedcom::Record - a module to manipulate Gedcom records
 
-Version 1.05 - 20th July 1999
+Version 1.06 - 13th February 2000
 
 =head1 SYNOPSIS
 
   use Gedcom::Record;
 
-  $self->parse($record, $grammar)
-  $record->collect_xrefs($callback)
-  my $xref = $self->resolve_xref($self->{value})
-  my @famc = $self->resolve($self->child_values("FAMC"))
-  $record->resolve_xrefs($callback)
-  $record->unresolve_xrefs($callback)
-  return 0 unless $child->validate_semantics
-  $record->normalise_dates($format)
-  $record->renumber($args)
-  my $child = $record->child_value("NAME")
-  my @children = $record->child_values("CHIL")
-  print $record->summary, "\n"
+  my $record  = tag_record("CHIL", 2);
+  my @records = tag_record("CHIL");
+  my @recs = $record->record("birth");
+  my @recs = $record->record("birth", "date");
+  my $rec  = $record->record("birth date");
+  my $rec  = $record->record(["birth", 2], "date");
+  my @recs = $record->get_record("birth");
+  my $val  = $record->get_value;
+  my @vals = $record->get_value("date");
+  my @vals = $record->get_value("birth", "date");
+  my $val  = $record->get_value("birth date");
+  my $val  = $record->get_value(["birth", 2], "date");
+  $self->parse($record, $grammar);
+  $record->collect_xrefs($callback);
+  my $xref = $record->resolve_xref($record->{value});
+  my @famc = $record->resolve $record->get_value("FAMC");
+  $record->resolve_xrefs($callback);
+  $record->unresolve_xrefs($callback);
+  return 0 unless $record->validate_semantics;
+  $record->normalise_dates($format);
+  $record->renumber($args);
+  print $record->summary, "\n";
+  $record->delete_record($sub_record);
 
 =head1 DESCRIPTION
 
@@ -388,51 +470,107 @@ Used by renumber().
 
 =head1 METHODS
 
+=head2 tag_record
+
+  my $record  = tag_record("CHIL", 2);
+  my @records = tag_record("CHIL");
+
+Get specific sub-records from the record.  This function is identical to
+Gedcom::Item::get_item().
+
+The arguments are the name of the tag, and optionally the count.
+
+In scalar context, returns the sub-record, or undef if it doesn't exist.
+In array context, returns all sub-records matching the specified tag.
+
+=head2 record
+
+  my @recs = $record->record("birth");
+  my @recs = $record->record("birth", "date");
+  my $rec  = $record->record("birth date");
+  my $rec  = $record->record(["birth", 2], "date");
+  my @recs = $record->get_record("birth");
+
+Retrieve a record.
+
+The get_record() function is identical to the record() function.
+
+In scalar context, record() returns the specified record, or undef if
+there is none.  In list context, record() returns all the specified
+records.
+
+Records may be specified by a list of strings.  Each string is either a
+Gedcom tag or a description.  Starting from the first string in the
+list, specified records are retrieved.  Then from those records, records
+specified by the next string in the list are retrieved.  This continues
+until all strings from the list have been used.
+
+In list context, all specified records are retrieved.  In scalar
+context, only the first record is retrieved.  If a record other than the
+first is wanted, then instead of passing a string, a reference to an
+array containing the string and a count may be passed.
+
+Instead of specifying a list of strings, it is possible to specify a
+single space separated string.  This can make the interface nicer.
+
+=head2 get_value
+
+  my $val  = $record->get_value;
+  my @vals = $record->get_value("date");
+  my @vals = $record->get_value("birth", "date");
+  my $val  = $record->get_value("birth date");
+  my $val  = $record->get_value(["birth", 2], "date");
+
+Retrieve a record's value.
+
+If arguments are specified, record() is first called with those
+arguments, and the values of those records are returned.
+
 =head2 parse
 
-  $self->parse($record, $grammar)
+  $self->parse($record, $grammar);
 
 Parse a Gedcom record.
 
 Match a Gedcom::Record against a Gedcom::Grammar.  Warn of any
 mismatches, and associate the Gedcom::Grammar with the Gedcom::Record as
-$self->{grammar}.  Do this recursively.
+$record->{grammar}.  Do this recursively.
 
 =head2 collect_xrefs
 
-  $record->collect_xrefs($callback)
+  $record->collect_xrefs($callback);
 
 Recursively collect all the xrefs.  Called by Gedcom::collect_xrefs.
 $callback is not used yet.
 
 =head2 resolve_xref
 
-  my $xref = $self->resolve_xref($value)
+  my $xref = $record->resolve_xref($value);
 
 See Gedcom::resolve_xrefs()
 
 =head2 resolve
 
-  my @famc = $self->resolve $self->child_values("FAMC")
+  my @famc = $record->resolve $record->tag_value("FAMC");
 
 For each argument, either return it or, if it an xref, return the
 referenced record.
 
 =head2 resolve_xrefs
 
-  $record->resolve_xrefs($callback)
+  $record->resolve_xrefs($callback);
 
 See Gedcom::resolve_xrefs()
 
 =head2 unresolve_xrefs
 
-  $record->unresolve_xrefs($callback)
+  $record->unresolve_xrefs($callback);
 
 See Gedcom::unresolve_xrefs()
 
 =head2 validate_semantics
 
-  return 0 unless $child->validate_semantics
+  return 0 unless $record->validate_semantics;
 
 Validate the semantics of the Gedcom::Record.  This performs a number of
 consistency checks, but could do even more.
@@ -441,7 +579,7 @@ Returns true iff the Record is valid.
 
 =head2 normalise_dates
 
-  $record->normalise_dates($format)
+  $record->normalise_dates($format);
 
 Change the format of all dates in the record.
 
@@ -449,7 +587,7 @@ See the documentation for Gedcom::normalise_dates
 
 =head2 renumber
 
-  $record->renumber($args)
+  $record->renumber($args);
 
 Renumber the record.
 
@@ -457,29 +595,33 @@ See Gedcom::renumber().
 
 =head2 child_value
 
-  my $child = $record->child_value("NAME")
+NOTE - This function is deprecated - use tag_value instead.
 
-Return the value of the specified child, or undef if the child could not
-be found.  Calls get_child().
+  my $child = $record->child_value("NAME");
 
 =head2 child_values
 
-  my @children = $record->child_values("CHIL")
+NOTE - This function is deprecated - use tag_value instead.
 
-Return a list of the values of the specified children.  Calls
-get_children().
+  my @children = $record->child_values("CHIL");
 
 =head2 summary
 
-  print $record->summary, "\n"
+  print $record->summary, "\n";
 
 Return a line of text summarising the record.
 
+=head2 delete_record
+
+  $record->delete_record($sub_record);
+
+Delete the specified sub-record from the record.
+
 =head2 Access functions
 
 All the Gedcom tag names can be used as function names.  Depending on
 the context in which they are called, the functions return either an
-array of the specified children, or the first specified child.
+array of the specified sub-items, or the first specified sub-item.
 
 The descriptions of the tags, with spaces replaced by underscores, can
 also be used as function names.  The function names can be of either, or
@@ -490,18 +632,18 @@ will need to qualify it or C<use subs>.
 =cut
 
 =begin if we cannot make the min/max assumptions specified in
-       Gedcom::Grammar::valid_children
+       Gedcom::Grammar::valid_items
 
 use as:
-  my @children = @{$ged->{record}{children}};
+  my @items = @{$ged->{record}{items}};
   my ($m, $w) =
-   $ged->{record}->validate_structure($ged->{record}{grammar}, \@children, 1);
+   $ged->{record}->validate_structure($ged->{record}{grammar}, \@items, 1);
   warn $w if $w;
 
 sub validate_grammar
 {
   my $self = shift;
-  my ($grammar, $children, $all) = @_;
+  my ($grammar, $items, $all) = @_;
   $I++;
   my $min = $grammar->min;
   my $max = $grammar->max;
@@ -514,13 +656,13 @@ sub validate_grammar
   {
     print " $value, $min -> $max\n" if $D;
     for (my $c = 0;
-         $c <= $#$children && ($all == 1 || !$max || $matches < $max);)
+         $c <= $#$items && ($all == 1 || !$max || $matches < $max);)
     {
-      if ($children->[$c]{tag} eq $value)
+      if ($items->[$c]{tag} eq $value)
       {
-        my $w = $children->[$c]->validate_syntax2;
+        my $w = $items->[$c]->validate_syntax2;
         $warn .= $w;
-        splice @$children, $c, 1;
+        splice @$items, $c, 1;
         $matches++;
       }
       else
@@ -540,7 +682,7 @@ sub validate_grammar
     my ($m, $w);
     do
     {
-      ($m, $w) = $self->validate_structure($s, $children, $all);
+      ($m, $w) = $self->validate_structure($s, $items, $all);
       if ($m)
       {
         $matches += $m;
@@ -555,20 +697,20 @@ sub validate_grammar
 sub validate_structure
 {
   my $self = shift;
-  my ($structure, $children, $all) = @_;
+  my ($structure, $items, $all) = @_;
   $all = 0 unless defined $all;
   $I++;
   print "  " x $I . "validate_structure($structure->{structure}, $all)\n" if $D;
   my $warn = "";
   my $total_matches = 0;
-  for my $child (@{$structure->{children}})
+  for my $item (@{$structure->{items}})
   {
-    my $min = $child->min;
-    my $max = $child->max;
-    my ($matches, $w) = $self->validate_grammar($child, $children, $all);
+    my $min = $item->min;
+    my $max = $item->max;
+    my ($matches, $w) = $self->validate_grammar($item, $items, $all);
     $warn .= $w;
     my $file = $self->{gedcom}{record}{file};
-    my $value = $child->{tag} || $child->{structure}{structure};
+    my $value = $item->{tag} || $item->{structure}{structure};
     my $msg = "$file:$self->{line}: $self->{tag}" .
               (defined $self->{xref} ? " $self->{xref} " : "") .
               " has $matches $value" . ($matches == 1 ? "" : "s");
@@ -579,7 +721,7 @@ sub validate_structure
       {
         $warn .= "$msg - minimum is $min\n" if $matches < $min;
         $warn .= "$msg - maximum is $max\n" if $matches > $max && $max;
-        $total_matches += $matches;                  # only one child is allowed
+        $total_matches += $matches;                   # only one item is allowed
         last;
       }
     }
@@ -587,7 +729,7 @@ sub validate_structure
     {
       $warn .= "$msg - minimum is $min\n" if $matches < $min;
       $warn .= "$msg - maximum is $max\n" if $matches > $max && $max;
-      $total_matches = 1 if $matches;                # all children are required
+      $total_matches = 1 if $matches;                   # all items are required
     }
   }
   print "  " x $I . "returning $total_matches matches\n" if $D;
@@ -600,7 +742,7 @@ sub validate_syntax2
   my $self = shift;
   $self->{gedcom}{validate_callback}->($self)
     if defined $self->{gedcom}{validate_callback};
-  my $children = [ @{$self->{children}} ];
+  my $items = [ @{$self->{items}} ];
   $I++;
   my $grammar = $self->{grammar};
   print "  " x $I . "validate_syntax2($grammar->{tag})\n" if $D;
@@ -608,32 +750,32 @@ sub validate_syntax2
   my $file = $self->{gedcom}{record}{file};
   my $here = "$file:$self->{line}: $self->{tag}" .
              (defined $self->{xref} ? " $self->{xref}" : "");
-  for my $child (@$children)
+  for my $item (@$items)
   {
-    print "  " x $I . "level $child->{level} on $self->{level}\n" if $D;
-    $warn .= "$here: Can't add level $child->{level} to $self->{level}\n"
-      if $child->{level} > $self->{level} + 1;
+    print "  " x $I . "level $item->{level} on $self->{level}\n" if $D;
+    $warn .= "$here: Can't add level $item->{level} to $self->{level}\n"
+      if $item->{level} > $self->{level} + 1;
   }
-  for my $child (@{$grammar->{children}})
+  for my $item (@{$grammar->{items}})
   {
-    my $min = $child->min;
-    my $max = $child->max;
-    my ($matches, $w) = $self->validate_grammar($child, $children, 1);
+    my $min = $item->min;
+    my $max = $item->max;
+    my ($matches, $w) = $self->validate_grammar($item, $items, 1);
     $warn .= $w;
-    my $value = $child->{tag} || $child->{structure}{structure};
+    my $value = $item->{tag} || $item->{structure}{structure};
     my $msg = "$here has $matches $value" . ($matches == 1 ? "" : "s");
     print "  " x $I . "$msg - minimum is $min maximum is $max\n" if $D;
     $warn .= "$msg - minimum is $min\n" if $matches < $min;
     $warn .= "$msg - maximum is $max\n" if $matches > $max && $max;
   }
-  if (@$children)
+  if (@$items)
   {
-    my %tags = map { $_ => 1 } $grammar->children;
-    for my $c (@$children)
+    my %tags = map { $_ => 1 } $grammar->items;
+    for my $c (@$items)
     {
       my $tag = $c->{tag};
       my $msg = exists $tags{$tag} ? "an extra" : "not a";
-      $warn .= "$file:$c->{line}: $tag is $msg child of $self->{tag}\n"
+      $warn .= "$file:$c->{line}: $tag is $msg item of $self->{tag}\n"
         unless $tag eq "CONT" || $tag eq "CONC" || substr($tag, 0, 1) eq "_";
     }
   }
